#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Dec 11 20:05:15 2021

@author: m102324
"""
import sys
import math
from cobindability.BED import bedtolist,unionBed3,subtractBed3
import logging
import numpy as np
import pandas as pd
from random import sample

